from django.db import models
from django.utils.translation import ugettext_lazy as _

import pureftpd_admin.settings as ftpusets_settings


class FtpUsers(models.Model):
    username = models.CharField(_('User name'), max_length=16, unique=True)
    is_active = models.BooleanField(_('Is active'), default=True)
    password = models.CharField(_('Password'), max_length=64)
    uid = models.SmallIntegerField(_('Uid'), default=ftpusets_settings.DEFAULT_UID)
    gid = models.SmallIntegerField(_('Gid'), default=ftpusets_settings.DEFAULT_GID)
    path = models.TextField(_('Directory on server'))
    upload_bandwidth = models.PositiveSmallIntegerField(_('Upload bandwidth'),
                                                        default=ftpusets_settings.DEFAULT_UPLOAD_BANDWIDTH,
                                                        help_text=_(
                                                            "MySQLGetBandwidthUL is the upload bandwidth restrictions. Values should be in KB/s."))
    download_bandwidth = models.PositiveSmallIntegerField(_('Download bandwidth'),
                                                          default=ftpusets_settings.DEFAULT_DOWNLOAD_BANDWIDTH,
                                                          help_text=_(
                                                              "MySQLGetBandwidthDL is the download bandwidth restrictions. Values should be in KB/s."))
    upload_ratio = models.PositiveSmallIntegerField(_('Upload ratio'), default=ftpusets_settings.DEFAULT_UPLOAD_RATIO,
                                                    help_text=_("MySQLGetRatioUL is the ratio upload/download."))
    download_ratio = models.PositiveSmallIntegerField(_('Download ratio'),
                                                      default=ftpusets_settings.DEFAULT_DOWNLOAD_RATIO,
                                                      help_text=_("MySQLGetRatioDL is the ratio download/upload."))
    description = models.TextField(_('Description'), blank=True)
    ipaccess = models.CharField(_('Ip access'), max_length=15, default=ftpusets_settings.DEFAULT_IPACCESS)
    quota_files = models.PositiveIntegerField(_('Quota files (Num)'), default=ftpusets_settings.DEFAULT_QUOTA_FILES,
                                              help_text=_(
                                                  'MySQLGetQTAFS is the maximal number of files a user can store in his home directory.'))
    quota_size = models.PositiveSmallIntegerField(_('Quota size (Mb)'), default=ftpusets_settings.DEFAULT_QUOTA_SIZE,
                                                  help_text=_('MySQLGetQTASZ is the maximal disk usage, in Megabytes.'))

    def get_uri(self):
        return "ftp://%(username)s:%(password)s@%(hostname)s" % {'username': self.username,
                                                                 'password': self.password,
                                                                 'hostname': ftpusets_settings.DEFAULT_HOSTNAME}

    def display_link_uri(self):
        uri = self.get_uri()
        return "<a target='_blank' href='%s'>%s</a>" % (uri, uri)

    display_link_uri.allow_tags = True

    class Meta:
        verbose_name = 'Ftp user'
        verbose_name_plural = 'Ftp users'
        db_table = 'ftpusers'




from django.conf import settings
from django.core.urlresolvers import reverse
from django.http.response import HttpResponseRedirect
from django.shortcuts import render_to_response

import os, posixpath

try:
    from urllib.parse import unquote
except ImportError:     # Python 2
    from urllib import unquote

def directory_index(path, fullpath):
    show_hidden = False
    nodes = (node for node in os.listdir(fullpath))
    if not show_hidden:
        nodes = filter(lambda x: not x.startswith('.'), nodes)

    files =  sorted(filter(lambda f: os.path.isfile(os.path.join(fullpath, f)), nodes), key=lambda x: x.lower())
    dirs = map(lambda d: d+'/',
               sorted(filter(lambda d: d not in files and os.access(os.path.join(fullpath, d), os.R_OK), nodes),
                      key=lambda x: x.lower()))
    context = {
        'directory' : path + '/',
        'dirs' : dirs,
        'files' : files,
        'fullpath': '' if fullpath == os.sep else fullpath, # if path is root
        'STATIC_URL': settings.STATIC_URL,
    }
    return render_to_response('localdir/directory_index.html', context)

def serve(request, path, document_root=None):
    path = posixpath.normpath(unquote(path))
    path = path.lstrip('/')
    newpath = ''
    for part in path.split('/'):
        if not part:
            # Strip empty path components.
            continue
        drive, part = os.path.splitdrive(part)
        head, part = os.path.split(part)
        if part in (os.curdir, os.pardir):
            # Strip '.' and '..' in path.
            continue
        newpath = os.path.join(newpath, part).replace('\\', '/')
    if newpath and path != newpath:
        return HttpResponseRedirect(newpath)

    fullpath = newpath and os.path.join(document_root, newpath) or os.path.normpath(document_root)

    if os.path.isdir(fullpath):
        return directory_index(newpath, fullpath)
    return HttpResponseRedirect('../?'+ request.GET.urlencode())


__author__ = 'ivan'

from pureftpd_admin.settings import *

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'pureftpd',                      # Or path to database file if using sqlite3.
        # The following settings are not used with sqlite3:
        'USER': 'pureftpd',
        'PASSWORD': '',
        'HOST': '',                      # Empty for localhost through domain sockets or '127.0.0.1' for localhost through TCP.
        'PORT': '',                      # Set to empty string for default.
    }
}

USE_X_FORWARDED_HOST = True
STATIC_URL = '/pureftpd_admin/static/'
STATIC_ROOT = '/pureftpd-admin/files/static'
MEDIA_URL = '/pureftpd_admin/media/'
MEDIA_ROOT = '/pureftpd-admin/files/media'

FTPUSERS_DEFAULT_UID = 81
FTPUSERS_DEFAULT_GID = 81
FTPUSERS_ROOT_PATH = '/www'


from django.forms import ModelForm, ValidationError
from django.contrib.admin.widgets import AdminTextInputWidget
from django.core.urlresolvers import reverse_lazy
from django.utils.crypto import get_random_string

try:
    from pureftpd_admin.localdir.widgets import LocalDirAdminWidget as PathWidget
except ImportError:
    from django.forms import Textarea as PathWidget

from pureftpd_admin.models import FtpUsers
import pureftpd_admin.settings as ftpusets_settings

__author__ = 'ivan'


class FtpUsersAdminForm(ModelForm):

    def __init__(self, *args, **kwargs):
        if not kwargs.has_key('instance'):
            kwargs['initial'] = {'password': get_random_string(6)}
        super(FtpUsersAdminForm, self).__init__(*args, **kwargs)

        self.fields["username"].widget.attrs.update({
            "autocomplete": "off"
        })
        self.fields["password"].widget.attrs.update({
            "autocomplete": "off"
        })

    def clean_password(self):
        data = self.cleaned_data['password']
        if len(data) < 6:
            raise ValidationError("Password is very simple, minimal must be 6 characters!")
        return data

    class Meta:
        model = FtpUsers
        widgets = {
            'path': PathWidget(attrs={'popup_url': reverse_lazy('localdir-ftpusers-url', kwargs={'path': ''}),
                                      'document_root': ftpusets_settings.ROOT_PATH}),
            'password': AdminTextInputWidget(attrs={'autocomplete': 'off'}),
            'username': AdminTextInputWidget(attrs={'autocomplete': 'off'}),
        }


__author__ = 'ivan'

from django.conf import settings
from django.contrib import admin

from .forms import FtpUsersAdminForm
from .models import FtpUsers

class FtpUsersAdmin(admin.ModelAdmin):
    search_fields = ['username', 'path']
    list_display = ['username', 'password', 'display_link_uri', 'uid', 'gid', 'path', 'is_active']
    list_editable = ['is_active']
    list_filter = ['uid', 'gid', 'is_active',
                   'upload_bandwidth', 'download_bandwidth',
                   'upload_ratio', 'download_ratio', 'ipaccess',
                   'quota_files', 'quota_size']
    form = FtpUsersAdminForm
    list_max_show_all = 500

    fieldsets = (
        (None, {
            'fields': ('username', 'password', 'uid', 'gid', 'path','is_active',)
        }),
        ('Restriction options', {
            'classes': ('collapse',),
            'fields': ('upload_bandwidth', 'download_bandwidth', 'upload_ratio', 'download_ratio', 'ipaccess', 'quota_size', 'quota_files')
        }),
        (None, {
            'fields': ('description',)
        }),
    )

    class Media:
        js = (settings.STATIC_URL + 'localdir/js/script.js',)
        css = {
            'all': (settings.STATIC_URL + 'localdir/css/styles.css',)
        }

admin.site.register(FtpUsers, FtpUsersAdmin)



__author__ = 'ivan'

from django.conf import settings
import platform

DEFAULT_UID = getattr(settings, 'FTPUSERS_DEFAULT_UID', -1)
DEFAULT_GID = getattr(settings, 'FTPUSERS_DEFAULT_GID', -1)
DEFAULT_UPLOAD_BANDWIDTH = getattr(settings, 'FTPUSERS_DEFAULT_UPLOAD_BANDWIDTH', 0)
DEFAULT_DOWNLOAD_BANDWIDTH = getattr(settings, 'FTPUSERS_DEFAULT_DOWNLOAD_BANDWIDTH', 0)
DEFAULT_UPLOAD_RATIO = getattr(settings, 'FTPUSERS_DEFAULT_UPLOAD_RATIO', 0)
DEFAULT_DOWNLOAD_RATIO = getattr(settings, 'FTPUSERS_DEFAULT_DOWNLOAD_RATIO', 0)
DEFAULT_QUOTA_SIZE = getattr(settings, 'FTPUSERS_DEFAULT_QUOTA_SIZE', 0)
DEFAULT_QUOTA_FILES = getattr(settings, 'FTPUSERS_DEFAULT_QUOTA_FILES', 0)
DEFAULT_IPACCESS = getattr(settings, 'FTPUSERS_DEFAULT_IPACCESS', "*")
DEFAULT_HOSTNAME = getattr(settings, 'FTPUSERS_DEFAULT_HOSTNAME', platform.node())

ROOT_PATH = getattr(settings, 'FTPUSERS_ROOT_PATH', '/')



__author__ = 'ivan'

from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django import forms

import os, re

popup_init_script = """
<br>
<a href="#" onclick="popupWin = window.open(django.jQuery('#%(target_id)s').attr('popup_url')+'?pop=1&id=%(target_id)s', 'Localdir popup', 'location,width=400,height=800,top=100,left=500'); popupWin.focus();">
    %(select_text)s
</a>
"""

class LocalDirAdminWidget(forms.Textarea):
    def render(self, name, value, attrs=None):
        if value is None: value = ''
        try:
            popup_url = str(self.attrs['popup_url'])
        except KeyError:
            raise TypeError("You need set widget's attr 'popup_url' for LocalDirAdminWidget")

        document_root = self.attrs.pop('document_root', '')
        if value.startswith(document_root):
            popup_url = os.path.normpath('/'.join([popup_url, re.sub(document_root + '/?', '', value, count=1)])) + '/'
            print popup_url
            self.attrs['popup_url'] = popup_url

        content = super(LocalDirAdminWidget, self).render(name, value, attrs)
        content += mark_safe(popup_init_script % {
            'target_id': attrs['id'],
            'select_text': unicode(_("Chose from local dir")),
            'popup_url': popup_url})
        return content





# -*- coding: utf-8 -*-
__author__ = 'ivan'


from django.db.models.loading import get_model
from django.db.models.query_utils import Q
from django.utils.datastructures import SortedDict
from django.db import connections
from django.core.management.base import BaseCommand, CommandError

from optparse import make_option

FtpUsers = get_model(app_label='pureftpd_admin', model_name='ftpusers')


class Command(BaseCommand):
    help = """Write to stdout example config for pureftpd when using mysql authentication

Add this argument in command line's run pureftpd server '-l mysql:/etc/pureftpd-mysql.conf'

INFO:

\L is replaced by the login of a user trying to authenticate.
\I is replaced by the IP address the client connected to.
\P is replaced by the port number the client connected to.
\R is replaced by the remote IP address the client connected from.
\D is replaced by the remote IPv4 address, as a long decimal number.

You will be comment some line if you don't want to use it."""


    def __init__(self):
        super(Command, self).__init__()
        self.option_list += (
            make_option('--socket', help="Mysql socket"),
            make_option('--address', dest="ADDRESS:PORT", help="Mysql address for connection, default is 'localhost:3306'"),
            make_option('--database-alias', default='default', help="Django database alias from DJANGO_SETTINGS_MODULE"),
            make_option('--any', action="store_true", default=False, help="Use not only cleartext password if field"),
        )

    def handle(self, *args, **options):
        context = SortedDict({})

        if options['socket']:
            context['MYSQLSocket'] = options['socket']
        elif options['ADDRESS:PORT']:
            context['MYSQLServer'], context['MYSQLPort'] = options['ADDRESS:PORT'].split(':')
        else:
            context['MYSQLServer'], context['MYSQLPort'] = ('localhost', '3306')

        if not options['any']:
            context['MYSQLCrypt'] = 'cleartext'

        try:
            connection = connections[options['database_alias']]
        except:
            raise CommandError("Can't find db or user for specified database alias: '%s'" % options['database_alias'])

        context['MYSQLDatabase'] = connection.settings_dict['NAME']
        context['MYSQLUser'] = connection.settings_dict['USER']
        context['MYSQLPassword'] = connection.settings_dict['PASSWORD']


        context['MYSQLGetPW'] = FtpUsers.objects.filter(username='"\L"', is_active=True).filter(Q(ipaccess='"*"') | Q(ipaccess='"\R"'))\
            .only('password').values_list('password', flat=True).query
        context['MYSQLGetUID'] = FtpUsers.objects.filter(username='"\L"')\
            .only('uid').values_list('uid', flat=True).query
        context['MYSQLGetGID'] = FtpUsers.objects.filter(username='"\L"')\
            .only('gid').values_list('gid', flat=True).query
        context['MYSQLGetDir'] = FtpUsers.objects.filter(username='"\L"')\
            .only('path').values_list('path', flat=True).query

        #Quota
        context['MySQLGetQTAFS'] = FtpUsers.objects.filter(username='"\L"')\
            .only('quota_files').values_list('quota_files', flat=True).query
        context['MySQLGetQTASZ'] = FtpUsers.objects.filter(username='"\L"')\
            .only('quota_size').values_list('quota_size', flat=True).query
        #Ratio
        context['MySQLGetRatioUL'] = FtpUsers.objects.filter(username='"\L"')\
            .only('upload_ratio').values_list('upload_ratio', flat=True).query
        context['MySQLGetRatioDL'] = FtpUsers.objects.filter(username='"\L"')\
            .only('download_ratio').values_list('download_ratio', flat=True).query
        #Bandwidth
        context['MySQLGetBandwidthUL'] = FtpUsers.objects.filter(username='"\L"')\
            .only('upload_bandwidth').values_list('upload_bandwidth', flat=True).query
        context['MySQLGetBandwidthDL'] = FtpUsers.objects.filter(username='"\L"')\
            .only('download_bandwidth').values_list('download_bandwidth', flat=True).query

        for option, value in context.iteritems():
            print option, '\t', value




from django.conf.urls import patterns, url
from django.contrib.admin.views.decorators import staff_member_required

__author__ = 'ivan'

from pureftpd_admin.localdir.views import serve
import pureftpd_admin.settings as ftpusets_settings

urlpatterns = patterns('',
                       url(r'^localdir/(?P<path>.*)$', staff_member_required(serve),
                           kwargs={'document_root': ftpusets_settings.ROOT_PATH}, name='localdir-ftpusers-url')
)



import os

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

config = {
    'description': 'This is the web base administration interface for pureftpd server',
    'long_description': README,
    'author': 'Ivan Fedoseev',
    'url': 'https://ivanff@bitbucket.org/ivanff/django-pureftpd-admin',
    'author_email': 'agestart@gmail.com',
    'version': '0.1.3a2',
    'install_requires': ['Django', 'Mysql-python'],
    'packages': find_packages(),
    'include_package_data': True,
    'name': 'django-pureftpd-admin',
    'classifiers': [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    'platforms': ['any'],
}

setup(**config)